/* @(#)70       1.2  src/gos/2d/XTOP/X11/shrmem.h, shrmem_mgr, gos411, 9428A410i 2/10/94 16:48:17 */
/*
 * COMPONENT_NAME: X11
 *
 * FUNCTIONS: none
 *
 * ORIGINS: 27
 *
 * (C) COPYRIGHT International Business Machines Corp. 1989, 1990
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef SHRMEM_H
#define SHRMEM_H

#define shmOffsetToPointer(base,offset) \
    ((unsigned int)(offset)+(char*)(base))

#define shmPointerToOffset(base,pointer) \
    ((char*)(pointer)-(unsigned int)(base))

#endif

